import uuid
import cv2
import os

import instance_seg.logging_util as logging_util

logger = logging_util.get_logger(os.path.basename(__file__).split('.')[0])

def create_new_txt(image , label ,  points,train_path_images , train_path_labels,dict):
    try:
        save_name = uuid.uuid4()
        
        if type(image) == str:
            image = cv2.imread(image)

        cv2.imwrite(f'{train_path_images}/{save_name}.jpg',image)
        txt_path=f'{train_path_labels}/{save_name}.txt'
        
        new_height , new_width , c = image.shape
    
        for p in points: 
            labels = p.label 
            label = dict[labels]
            new_name = str(label) + ' '
            

            for ps in p: 
                x , y = list(ps)
                normalize_X = x/new_width
                normalize_Y = y/new_height
                new_name += f'{normalize_X:.6f} {normalize_Y:.6f}'
                new_name += ' '

            with open(txt_path,'a+') as f:
                
                f.write(new_name)
                f.write('\n')
                
                    
            del new_name
        
    except Exception as e:
        logger.warning(f'problem : create new text  desc : {e}')
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
            
    #         # print(x/new_width,y/new_height)
    #         # a = (x/new_width).ravel()
            
         
        
    #     print('***') 
        # print(points)
    
        # sh_dict={"label": label,"points":points , "group_id": None, "shape_type": "polygon", "flags": {}} 
        # annot_dict["shapes"].append(sh_dict)
        # annot_dict["imagePath"] = f'{save_name}.jpg'
        
        # img_data = labelme.LabelFile.load_image_file(f'{train_path_images}/{save_name}.jpg')
        # img_data = base64.b64encode(img_data).decode('utf-8')
        # height , width , _ = image.shape
        # annot_dict["imageData"]=img_data
        # annot_dict["imageHeight"]=height
        # annot_dict["imageWidth"]=width
        # # print(train_path_labels)
        # json_file=f'{train_path_labels}/{save_name}.json'
                        
        # with open(json_file, 'w', encoding='utf-8') as f:
        #         json.dump(annot_dict, f, ensure_ascii=False, indent=4,cls=NumpyEncoder)


import cv2
import numpy as np
from skimage.util import random_noise
from PIL import Image , ImageEnhance
import albumentations as A
import numpy as np

import random
from typing import List , Tuple




#zoom
def clipped_zoom(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array,List[List],List]:
    '''
    This function will zoom in/out an image and it coordinations

    : param image        : accepts cv2.imread() image
    : param bboxes       : bounding boxes location
    : param category_ids : ids of each category ( its a dictionary)
    : return             : transformed image , its bounding boxes and their ids
    '''

    zoom = random.choice([0.7,0.8,0.9,1.1,1.2,1.3,1.4])
    shears = random.choice([None,3,-3])
    transform = A.Compose(
    [A.augmentations.geometric.transforms.Affine(shear=shears,p=1.0,scale=zoom)],
    bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

    transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

    return transformed['image'] , transformed['bboxes'] , transformed['category_ids']




#affine
def affine_transform(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array,List[List],List]:

    '''
    This function will apply affine transform on an image and it coordinations

    : param image        : accepts cv2.imread() image
    : param bboxes       : bounding boxes location
    : param category_ids : ids of each category ( its a dictionary)
    : return             : transformed image , its bounding boxes and their ids
    '''

    affines = random.choice(range(-70,70,20))
    transform = A.Compose(
    [A.augmentations.geometric.transforms.ElasticTransform(alpha_affine=affines,p=.8,same_dxdy=True,)],
    bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

    transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

    return transformed['image'] , transformed['bboxes'] , transformed['category_ids']





#blurr
def blur(image:np.array) -> np.array:
    '''
        This function will blur images.
        Blur values are selected by random. (3,3) , (5,5)

        : param image : accepts cv2.imread() image
        : return      : blurred image
    '''
    blurrr = [(3,3),(5,5)]
    blurrr = random.choice(blurrr)
    return cv2.GaussianBlur(image ,blurrr,0)




#noise
def noise(image:np.array) -> np.array:

    '''
        This function will add noise to the images.
        Noise values are selected by random.

        : param image : accepts cv2.imread() image
        : return      : noisy image

    '''
    noise = [0.001,0.002,0.003,0.004]
    noise_1 = random.choice(noise)
    noise_img = random_noise(image, mode='s&p',amount=noise_1)
    noise_img = np.array(255*noise_img, dtype = 'uint8')
    return noise_img




#brightness-darkness
def brightness_contrast(image:np.array)-> Tuple[np.array, np.array]:

    '''
        This function will add brightness and darkness to the images.
        brightness/darkness values are selected by random.

        : param image : accepts cv2.imread() image
        : return      : brightness image , darkness image
    '''
    value = random.choice([10,20,30,40,50])
    value2 = random.choice([30,40,50,60,70,80,90])
    hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)

    h, s, v = cv2.split(hsv)
    v_bright = cv2.add(v,value)
    v_dark = cv2.add(v,-value2)
    final_hsv_bright = cv2.merge((h, s, v_bright))
    final_hsv_dark = cv2.merge((h, s, v_dark))

    img_bright = cv2.cvtColor(final_hsv_bright, cv2.COLOR_HSV2BGR)
    img_dark = cv2.cvtColor(final_hsv_dark, cv2.COLOR_HSV2BGR)
    return img_bright , img_dark




#noise_and_blur 
def noise_and_blur(image:np.array) -> np.array:

    '''
        This function will add noise and blur to the images.
        

        : param image : accepts cv2.imread() image
        : return      : noisy and blur image

    '''

    blurs = blur(image)
    noises = noise(blurs)
    return noises




#hue
def hue(image:np.array) -> np.array:

    '''
        This function will add hue to the images.
        Hue values are selected by random.

        : param image : accepts cv2.imread() image
        : return      : hue image

    '''

    hue_choice = random.choice(range(10,80))
    hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
    h,s,v = cv2.split(hsv)
    hnew = np.mod(h + int(hue_choice), 180).astype(np.uint8)
    hsv_new = cv2.merge([hnew,s,v])
    bgr_new = cv2.cvtColor(hsv_new, cv2.COLOR_HSV2BGR)
    return bgr_new




#image_translate
def image_translation(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array,List[List],List]:

    '''
    
    This function will apply translation on an image and it coordinations

    : param image        : accepts cv2.imread() image
    : param bboxes       : bounding boxes location
    : param category_ids : ids of each category ( its a dictionary)
    : return             : transformed image , its bounding boxes and their ids
    
    '''

    rotate = random.choice([0,1,2,3])
    zoom_out = random.choice([0.7,0.8,0.9])
    translate = random.choice([-50,40,-39,20,-10,10,20,30,40,50])
    transform = A.Compose(
    [A.augmentations.geometric.transforms.Affine(translate_px=translate,scale=zoom_out,rotate=rotate)],
    bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

    transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

    return transformed['image'] , transformed['bboxes'] , transformed['category_ids']




#saturation
def image_saturation(image:np.array) -> np.array:

    '''
        This function will add saturation to the images.
        Hue values are selected by random.

        : param image : accepts cv2.imread() image
        : return      : saturated image

    '''

    sat_level = random.choice([1,2,3])
    img = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    im_pil = Image.fromarray(img)
    converter = ImageEnhance.Color(im_pil)
    img2 = converter.enhance(sat_level)
    saturated_image = np.asarray(img2)
    saturated_image = cv2.cvtColor(saturated_image, cv2.COLOR_RGB2BGR)
    return saturated_image 


#contrast
def contrast(img:np.array) -> np.array:

    '''
        
        This function will add contrast to the images.
        contrast values are selected by random.

        : param image : accepts cv2.imread() image
        : return      : hue image

    '''

    level = random.choice(range(40,50))
    factor = (259 * (level + 255)) / (255 * (259 - level))
    def contrast(c):
        return 128 + factor * (c - 128)
    rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    im_pil = Image.fromarray(rgb)
    con = im_pil.point(contrast)
    bgr = np.asarray(con)
    contrast_im = cv2.cvtColor(bgr, cv2.COLOR_RGB2BGR)
    return contrast_im


#more aug on rotation
def more_aug(imagess:np.array) -> np.array:
    '''
        This function will randomly apply on rotated images (random rotation , +/- 90deg), to give it more complexity.
    '''
    more_augs = random.choice(['blur','noise','saturation','normal','contrast'])


    if more_augs == 'blur':
        return blur(imagess)
    if more_augs == 'noise':
        return noise(imagess)
    if more_augs == 'saturation':
        return image_saturation(imagess)
    if more_augs == 'contrast':
        return contrast(imagess)
    if more_augs == 'normal':
        return imagess





#normalize height and width for yolo
def normalize_yolo(image:np.array ,X_tl:int,Y_tl:int,b_width:int,b_height:int) -> Tuple[float,float,float,float]:

    '''
    This function will normalize coordinates values (200,100,...,) --> (0.002,0.444,...,)

    : param image              : it accepts cv2.imread()
    : param X_tl , Y_tl        : top left corner
    : param b_width , b_height : height and width of the bounding boxes

    '''

    image_height ,image_width , c = image.shape
    X_tl = (X_tl+(b_width/2))/image_width
    Y_tl = (Y_tl+(b_height/2))/image_height
    b_width = b_width/image_width
    b_height = b_height/image_height

    return round(X_tl,6) , round(Y_tl,6) , round(b_width,6) , round(b_height,6)




def vertical_flip(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array,List[List],List]:
    '''
    This function will rotate images 180degree

    : param image        : accepts cv2.imread() image
    : param bboxes       : bounding boxes location
    : param category_ids : ids of each category ( its a dictionary)
    : return             : transformed image , its bounding boxes and their ids

    '''

    transform = A.Compose(
    [A.augmentations.geometric.transforms.VerticalFlip(p=1),
    A.augmentations.geometric.transforms.HorizontalFlip(p=1)],
    bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

    transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

    return transformed['image'] , transformed['bboxes'] , transformed['category_ids']




#it will distort images little bit
# def GridDistortion(image,bboxes,category_ids): 
#     distort = random.choice([0.2,0.3])
#     transform = A.Compose(
#     [A.augmentations.geometric.transforms.GridDistortion(distort_limit=distort,normalized=True,p=1)],
#     bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

#     transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

#     return transformed['image'] , transformed['bboxes'] , transformed['category_ids']        













import cv2
import matplotlib.pyplot as plt
import numpy as np
from rich.console import Console

import os
import random
import shutil


BOX_COLOR = (255, 0, 0) # Red
TEXT_COLOR = (255, 255, 255)

console = Console()


def image_resize(img_path:os.path,image_size:int=416) -> np.array:
  '''
  This function will resize each images of given image_size.

  : param img_path      : provide image path.
  : param image_size    : image resizing value
  : return resized_image: It will return resized image
  '''
  try:
    output_size = image_size
    im_pth = img_path
    cv2_image = cv2.imread(im_pth)
    old_size = cv2_image.shape[:2] 
    ratio = float(output_size)/max(old_size)
    new_size = tuple([int(x*ratio) for x in old_size])
    resized_image = cv2.resize(cv2_image , (new_size[1], new_size[0]))
    return resized_image
  except Exception as e:
    pass




def train_test_split(folder:os.path,split:float=0.20) -> None:
  '''
  This function will divide your data into train_test.

  : param folder : provide folder name
  : param split  : provide split ratio
  
  '''
  try:
    all_images =  os.listdir(f'{folder}/images')
    len_total_images = len(all_images)
    split = int(len_total_images*split)
    sample = random.sample(all_images,split)
        
    if not os.path.exists(f'{folder}/train') or not os.path.exists(f'{folder}/test'):

            os.makedirs(f'{folder}/train/images')
            os.makedirs(f'{folder}/train/labels')
            if float(split) != 0.0:
              os.makedirs(f'{folder}/test/images')
              os.makedirs(f'{folder}/test/labels')
              
    if float(split) != 0.0:
      for images_ in sample:
          shutil.move(f'{folder}/images/{images_}',f'{folder}/test/images')
          text = images_.split('.')[0]
          shutil.move(f'{folder}/labels/{text}.txt',f'{folder}/test/labels')

    all_images =  os.listdir(f'{folder}/images')

    for images_ in all_images:
        shutil.move(f'{folder}/images/{images_}',f'{folder}/train/images')
        text = images_.split('.')[0]
        shutil.move(f'{folder}/labels/{text}.txt',f'{folder}/train/labels')


    shutil.rmtree(f'{folder}/images')
    shutil.rmtree(f'{folder}/labels')

  except Exception as e:
    pass







def folder_creation(folder_name:str) -> str:

    '''
    This function will create a folder for augmentations , where results will be saved
    
    '''
    try:
        saved_folder_name = folder_name 

        if not os.path.exists(f'{saved_folder_name}/images') or not os.path.exists(f'{saved_folder_name}/labels'):
            os.makedirs(f'{saved_folder_name}/images')
            os.makedirs(f'{saved_folder_name}/labels')
            console.print(f'[bold blue]{saved_folder_name}/[bold blue] - created')

        return saved_folder_name
    except Exception as e:
        raise RuntimeWarning(e)


def random_num():
    '''
    This function will provide random value for bounding box color / text
    '''
    rect = random.choice(range(1,256))
    rect1 = random.choice(range(1,256))
    rect2 = random.choice(range(1,256))
    return rect , rect1 , rect2


def yaml_writer(total_no_of_detection , labels_list,augmentation_folder_name):

    template = '''
# ImageAugs Support augmentation for Object detection , Instance segmentation and Classification.
 
train: ../train/images
test: ../test/images
val: ../test/images

nc: {}
names: {}


ImageAugs:
    type: Detection
    version: 1.0.1
        
    '''

    template_str = template.format(total_no_of_detection,labels_list)
    with open(f"{augmentation_folder_name}/data.yaml", "w") as f:
        f.write(template_str)

# def visualize_bbox(img, bbox, class_name, color=BOX_COLOR, thickness=2):
#     """Visualizes a single bounding box on the image"""
#     x_min, y_min, w, h = bbox
#     x_min, x_max, y_min, y_max = int(x_min), int(x_min + w), int(y_min), int(y_min + h)
   
#     cv2.rectangle(img, (x_min, y_min), (x_max, y_max), color=color, thickness=thickness)
    
#     ((text_width, text_height), _) = cv2.getTextSize(class_name, cv2.FONT_HERSHEY_SIMPLEX, 0.35, 1)    
#     cv2.rectangle(img, (x_min, y_min - int(1.3 * text_height)), (x_min + text_width, y_min), BOX_COLOR, -1)
#     cv2.putText(
#         img,
#         text=class_name,
#         org=(x_min, y_min - int(0.3 * text_height)),
#         fontFace=cv2.FONT_HERSHEY_SIMPLEX,
#         fontScale=0.35, 
#         color=TEXT_COLOR, 
#         lineType=cv2.LINE_AA,
#     )
#     return img


# def visualize(image, bboxes, category_ids, category_id_to_name):
#     img = image.copy()
#     for bbox, category_id in zip(bboxes, category_ids):
#         class_name = category_id_to_name[category_id]
#         img = visualize_bbox(img, bbox, class_name)
#     plt.figure(figsize=(12, 12))
#     plt.axis('off')
#     plt.imshow(img)
#     plt.show()




    


from asyncio.log import logger
import bbox_visualizer as bbv
import pybboxes as pbx
import cv2
import numpy as np
from rich.console import Console

import pickle
import os

from object_detection_aug import utils_py as utils_py



console = Console()


def bbbox_viewer(image_path:os.path,id_pickle_file:os.path='pickle_files/ids.pickle') -> np.array :

    '''
        This function will draw bounding boxes.

        : param id_pickle_file : This function requires "ids.pickle" file, for accessing the id.
                              id pickle file saved in pickle_files/ids.pickle inside script folder
        
        : param image_path     : provide image path for draw bounding boxes from train / test folder

    '''
    
    try:
        console.print('[bold green]Pickle file found..[/bold green]')
        with open(id_pickle_file,'rb') as pick:
            ids = pickle.load(pick)
    
    except Exception as e:
        raise FileNotFoundError('Pickle file not found ! please provide full path..')
        

   
    try:
        coor = image_path.replace('.jpg','.txt')
        coor = coor.replace('images','labels')
        try:
            read_image = cv2.imread(image_path)
            h , w,c = read_image.shape

        except Exception as e:
            raise FileNotFoundError('Image file not found ! please provide full path..')
        

        with open(coor) as f:
            loc = f.read().splitlines()
            for cors in loc:
                
                id = cors.split()[0]
                coor = tuple(map(float,cors.split()[1:]))
                rect , rect1 , rect2 = utils_py.random_num()
                x_tl , y_tl , width , height = pbx.convert_bbox(coor, from_type="yolo", to_type="voc",image_size=(w,h)) 
                bboxs = (x_tl,y_tl,width,height)
                cv2.rectangle(read_image, (bboxs[0], bboxs[1]), (bboxs[2], bboxs[3]),(rect,rect1,rect2),2)
                bbv.add_label(read_image,ids[int(id)], bbox=bboxs, top=True,text_color=(rect,rect1,rect2),draw_bg=False)

        return read_image

    except Exception as e:
        logger.error(f'Some problem with bbbox_viewer , {e}')
        bbv.add_label(read_image,'issue_found', bbox=bboxs, top=True,text_color=(rect,rect1,rect2),draw_bg=False)
        return read_image


# if __name__ == '__main__':
#     count = 1
#     for i in os.listdir('github_sample/train/images'):
     
#         path = f'github_sample/train/images/{i}'
#         image = bbbox_viewer(path,'pickle_files/ids.pickle')
#         cv2.imshow(f'_zoom',image)
#         cv2.imwrite(f'{count}.jpg',image)
#         cv2.waitKey(0)
#         count += 1


  
   






import albumentations as A
import numpy as np

import random
from typing import List , Tuple




def rotate_90deg(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array , List[List] , List]:

    '''
    This function will add rotation to a fraction of images, suppose you have 100 images and you want to add rotation only
    on randomly selected 20 images, by passing fraction param = 0.2, it will randomly select 20 images from 
    dataset and apply rotation to it, for selecting all images you have to pass fraction = 1.0 

    : param coordinates : coordinates is a pickle file containing coordinates 
                          of a image. coordinates contain (image_id , class_id , x,y,w,h) 
                          (load the pickle file first)
    
    : param images      : images a pickle file containing key , value pair.
                          key id of image_id (0,1,2,..) , value is cv2.imread() image(np.array)

    : param fraction    : fraction is float. by giving fraction =0.5 , it will randomly selected 50% data from your
                          dataset and apply blur to it.

    : param folder      : folder name where you want to save your augmentations.

    : return            : transfomed image , boxes coor and their ids
    
    '''
    try:
        choices = random.choice([0,1])

        if choices == 0:

            transform = A.Compose(
            [A.augmentations.geometric.rotate.Rotate(p=1.0,limit=(-90,-90))],

            bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

            transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

            return transformed['image'] , transformed['bboxes'] , transformed['category_ids']

        else:

            transform = A.Compose(
            [A.augmentations.geometric.rotate.Rotate(p=1.0,limit=(90,90))],
            bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

            transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

            return transformed['image'] , transformed['bboxes'] , transformed['category_ids']
        
    except Exception as e:
        pass




def random_rotation(image:np.array,bboxes:List[List],category_ids:List) -> Tuple[np.array , List[List] , List]:

    '''
    For documentation and comments please, refer to rotate_90deg
    
    '''
    try:
        choices = random.choice([0,1])

        if choices == 0:

            val = random.choice(range(5,25))
            transform = A.Compose(
            [A.augmentations.geometric.rotate.Rotate(p=1.0,limit=(val,val))],

            bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

            transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

            return transformed['image'] , transformed['bboxes'] , transformed['category_ids']

        else:
            val = random.choice(range(-25,-5))
            transform = A.Compose(
            [A.augmentations.geometric.rotate.Rotate(p=1.0,limit=(val,val))],
            bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

            transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

            return transformed['image'] , transformed['bboxes'] , transformed['category_ids']
        
    except Exception as e:
        pass


from configparser import ConfigParser

import os

import instance_seg.logging_util as logging_util


#logging
logger = logging_util.get_logger(os.path.basename(__file__))

def read_config(file_name:os.path) -> object: 

    '''
    This function will read config file
    
    : param read_config : it accepts config file full path
    : return            : it returns read config file
    
    '''

    
    config = ConfigParser()

    try:

        with open(file_name) as fh:

            config.read_file(fh)
            
        return config

    except Exception as e:

        logger.error(f'ConfigFile not found!,{e}')
        raise FileNotFoundError(f'Config file not found, Please provide full path of config file,{e}')
        







import uuid
import cv2
from tornado import concurrent
from tqdm import tqdm
import numpy as np
from rich.console import Console
from rich.table import Table



import pickle
from typing import List 
import os
import time

from object_detection_aug import filereads_py as filereads_py
from object_detection_aug import rotations as rotations 
from object_detection_aug import utils_py as utils_py
from object_detection_aug import augmentations as aug
from object_detection_aug import logging_util as logging_util


'''
Combining all the functions together
'''

mark = '''# Augmentations..'''


#logging
logger = logging_util.get_logger(os.path.basename(__file__))

# https://rich.readthedocs.io/en/stable/introduction.html
console = Console()

#creating table using rich module
table = Table(title="[b]Data augmentations table[/b]",style='red')

table.add_column("Augmentation name", justify="right", style="cyan", no_wrap=True)
table.add_column("Augmented images / Total images", style="magenta")
table.add_column("Fraction used", justify="right", style="green")


def raw_image(coordinates:List[List],images:np.array,folder:os.path) -> None:

    '''
    This function will save RAW images and its coordinates

    : param coordinates : coordinates is a pickle file containing coordinates 
                          of a image. coordinates contain (image_id , class_id , x,y,w,h) 
                          (load the pickle file first)
    
    : param images      : images a pickle file containing key , value pair.
                          key id of image_id (0,1,2,..) , value is cv2.imread() image(np.array)

    : param folder      : folder name where you want to save your augmentations.

    : return            : None
    
    '''
    try:
        for datas in  coordinates:

                name = uuid.uuid4()
                image = images[datas[0][0]]
                for coor in range(len(datas)):
                    id  = int(datas[coor][1])
                    X  = int(datas[coor][2])
                    Y  = int(datas[coor][3])
                    W  = int(datas[coor][4])
                    H = int(datas[coor][5])
            
            # #     #normalize
                    coor = aug.normalize_yolo(image,X,Y,W,H)

        
                    with open(f'{folder}/labels/{name}.txt','a+') as f:
                        f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

                cv2.imwrite(f'{folder}/images/{name}.jpg',image)

    except Exception as e:
        logger.warning(f"Problem with raw images, {e}")

    

    

def blur_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None: 

    '''
    This function will add blur to a fraction of images, suppose you have 100 images and you want to add blur only
    on randomly selected 20 images, by passing fraction param = 0.2, it will randomly select 20 images from 
    dataset and apply blur to it, for selecting all images you have to pass fraction = 1.0 

    : param coordinates : coordinates is a pickle file containing coordinates 
                          of a image. coordinates contain (image_id , class_id , x,y,w,h) 
                          (load the pickle file first)
    
    : param images      : images a pickle file containing key , value pair.
                          key id of image_id (0,1,2,..) , value is cv2.imread() image(np.array)

    : param fraction    : fraction is float. by giving fraction =0.5 , it will randomly selected 50% data from your
                          dataset and apply blur to it.

    : param folder      : folder name where you want to save your augmentations.

    : return            : None
    
    '''
    
    try:
        console.print('[bold blue]Blur started..[bold blue]')
        #select fraction of data by passing fraction param , coordinates is loaded pickle file
        blur_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        #blur frc is list
        for datas in blur_frc:
            #getting the name
            name = uuid.uuid4()
            #getting the read image
            image = images[datas[0][0]]
            #apply blur to it
            blurs = aug.blur(image)
            #getting the id , x,y,w,h coordinates (raw coor)
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
 
                #normalizing those coordinates
                coor = aug.normalize_yolo(blurs,X,Y,W,H)

                #saving coor inside given folder
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            #saving image inside given folder
            cv2.imwrite(f'{folder}/images/{name}.jpg',blurs)

       

    except Exception as e:
        logger.warning(f"Problem with blur images, {e}")

    console.print('[bold green]Blur completed..[bold green]')


def noise_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Noise started..[bold blue]')
        noise_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        for datas in noise_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            noises = aug.noise(image)

            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
                coor = aug.normalize_yolo(noises,X,Y,W,H)
                
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            cv2.imwrite(f'{folder}/images/{name}.jpg',noises)

    except Exception as e:
        logger.warning(f"Problem with noise images, {e}")
        
    console.print('[bold green]Noise completed..[bold green]')
    



def noise_blur_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Noise and blur started..[bold blue]')
        NB_frc = filereads_py.fraction_data(coordinates,fraction=fraction)

        for datas in NB_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            nb = aug.noise_and_blur(image)

            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
                

                coor = aug.normalize_yolo(nb,X,Y,W,H)
                

                

                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            cv2.imwrite(f'{folder}/images/{name}.jpg',nb)

    except Exception as e:
        logger.warning(f"Problem with noise_and_blur images, {e}")

    console.print('[bold green]Noise and blur completed..[bold green]')
    

def hue_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Hue started..[bold blue]')
        hue_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        for datas in hue_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            hue = aug.hue(image)

            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
                

                coor = aug.normalize_yolo(hue,X,Y,W,H)
                

                

                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            cv2.imwrite(f'{folder}/images/{name}.jpg',hue)

    except Exception as e:
        logger.warning(f"Problem with hue images, {e}")
    
    console.print('[bold green]Hue completed..[bold green]')     
   


def saturation_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Saturation started..[bold blue]')
        saturation_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        for datas in saturation_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            sat = aug.image_saturation(image)

            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
                coor = aug.normalize_yolo(sat,X,Y,W,H)

                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            cv2.imwrite(f'{folder}/images/{name}.jpg',sat)

    except Exception as e:
        logger.warning(f"Problem with saturation images, {e}")

    console.print('[bold green]Saturation completed..[bold green]')

def contrast_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        contrast_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        console.print('[bold blue]Contrast started..[bold blue]')
        for datas in contrast_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            contrast = aug.contrast(image)

            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
                coor = aug.normalize_yolo(contrast,X,Y,W,H)
                
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

            cv2.imwrite(f'{folder}/images/{name}.jpg',contrast)

    except Exception as e:
        logger.warning(f"Problem with contrast images, {e}")

    console.print('[bold green]Contrast completed..[bold green]')


def bright_dark_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:

        brightness_darkness_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        console.print('[bold blue]Bright/dark started..[bold blue]')
        for datas in brightness_darkness_frc:
            name = uuid.uuid4()
            image = images[datas[0][0]]
            bright , dark = aug.brightness_contrast(image)
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
            
            
                coor = aug.normalize_yolo(bright,X,Y,W,H)
                

                with open(f'{folder}/labels/{name}_bright.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

                with open(f'{folder}/labels/{name}_dark.txt','a+') as f:
                    f.write(f'{id} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')
        

            cv2.imwrite(f'{folder}/images/{name}_bright.jpg',bright)
            cv2.imwrite(f'{folder}/images/{name}_dark.jpg',dark)

    except Exception as e:
        logger.warning(f"Problem with brightness_darkness images, {e}")

    console.print('[bold green]Bright/dark completed..[bold green]')



def rotate_90_im(coordinates:List[list],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        rotation_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        console.print('[bold blue]Rotation started..[bold blue]')
        
        for datas in rotation_frc:
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        

            rotated_image , coor , id = rotations.rotate_90deg(image,temp_coor,temp_id)

            #adding (hue/sat/contrast/blur to rotated image , if you want you can turn it off)
            rotated_image = aug.more_aug(rotated_image)
            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

    except Exception as e:
        logger.warning(f"Problem with rotation90 images, {e}")
    
    console.print('[bold green]Rotation completed..[bold green]')
    
       




def random_rotate_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        random_rotate_frc = filereads_py.fraction_data(coordinates,fraction=fraction)

        for datas in random_rotate_frc:
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        

            rotated_image , coor , id = rotations.random_rotation(image,temp_coor,temp_id)
            #adding (hue/sat/contrast/blur to rotated image , if you want you can turn it off)
            rotated_image = aug.more_aug(rotated_image)
            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

    except Exception as e:
        logger.warning(f"Problem with random rotation images, {e}")


    



def zoom_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Zoom started..[bold blue]')
        zoom_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
        for datas in zoom_frc:
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        
            rotated_image , coor , id = aug.clipped_zoom(image,temp_coor,temp_id)
            #adding (hue/sat/contrast/blur to rotated image , if you want you can turn it off)
            rotated_image = aug.more_aug(rotated_image)

            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')
    
    except Exception as e:
        logger.warning(f"Problem with zoom images, {e}")
        
    console.print('[bold green]Zoom completed..[bold green]')
    
   

def affine_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Affine started..[bold blue]')
        affine_frc = filereads_py.fraction_data(coordinates,fraction=fraction)

        
        for datas in tqdm(affine_frc,desc='Affine'):
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        

            rotated_image , coor , id = aug.affine_transform(image,temp_coor,temp_id)
        
            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])

                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

    except Exception as e:
        logger.warning(f"Problem with affine images, {e}")

    console.print('[bold green]Affine completed..[bold green]')
    
   



def translation_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Translation started..[bold blue]')
        translation_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
    
        for datas in translation_frc:
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        

            rotated_image , coor , id = aug.image_translation(image,temp_coor,temp_id)
            #adding (hue/sat/contrast/blur to rotated image , if you want you can turn it off)
            # rotated_image = aug.more_aug(rotated_image)
            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')
    except Exception as e:
        logger.warning(f"Problem with translation images, {e}")

    console.print('[bold green]Translation completed..[bold green]')
 




def vertical_im(coordinates:List[List],images:np.array,fraction:float,folder:os.path) -> None:

    '''
    For documentation and comments please, refer to blur_im function

    '''
    try:
        console.print('[bold blue]Vertical flip started..[bold blue]')
        translation_frc = filereads_py.fraction_data(coordinates,fraction=fraction)
    
        for datas in translation_frc:
            name = uuid.uuid4()
            temp_coor = []
            temp_id = []
            image = images[datas[0][0]]
            for coor in range(len(datas)):
                id  = int(datas[coor][1])
                X  = int(datas[coor][2])
                Y  = int(datas[coor][3])
                W  = int(datas[coor][4])
                H = int(datas[coor][5])
                temp_coor.append([X,Y,W,H])
                temp_id.append(id)

        

            rotated_image , coor , id = aug.vertical_flip(image,temp_coor,temp_id)
            #adding (hue/sat/contrast/blur to rotated image , if you want you can turn it off)
            rotated_image = aug.more_aug(rotated_image)
            cv2.imwrite(f'{folder}/images/{name}.jpg',rotated_image)

            for ids , coors in zip(id,coor):
                coor = aug.normalize_yolo(rotated_image,coors[0],coors[1],coors[2],coors[3])
                with open(f'{folder}/labels/{name}.txt','a+') as f:
                    f.write(f'{ids} {coor[0]} {coor[1]} {coor[2]} {coor[3]}\n')

    except Exception as e:
        logger.warning(f"Problem with vertical images, {e}")

    console.print('[bold green]Vertical flip completed..[bold green]')



def main(folder:os.path,test_split:float=0.10,raw_images_ok:bool=False,blurs:bool=False,blur_f:float=.6,noise:bool=False,noise_f:float=.6,NB:bool=False,NB_f:float=.6,hue:bool=False,hue_f:float=.6,sat:bool=False,sat_f:float=.6,bright:bool=False,bright_f:float=.6,
 contrast:bool=False,contrast_f:float=0.5,rotation:bool=False,rotation_f:float=0.8,zoom:bool=False,zoom_f:float=.7,affine:bool=False,affine_f:float=0.7,translation:bool=False,translation_f:float= 0.7,vertical_flip:bool=False,vertical_f:float=0.2):

    '''
    This main function combines all the sub functions. This function will load all the pickle files first,by giving
    a folder name , train test split , boolean and float values you will get your augmented data.

    : param folder                         : folder where you want to save your images, create it by folder creation function

    : param train_test_split               : it will divide your data into train test split, it accepts float values 

    : param raw_images_ok                  : It is bool , whether you want to save raw images or not.

    : param blur , noise ...,translation   : It is bool , which augmentation you want to use in your data.

    : param blur_f,zoom_f,..,translation_f : It is float , fraction of data you want to use for augmentations
    
    '''

    folder = utils_py.folder_creation(folder_name=folder)

    #type checking for inputs
    type_1 = {  'train_test_split':test_split,
                'blur_f':blur_f,
                'noise_f':noise_f,
                'NB_f':NB_f,
                'hue_f':hue_f,
                'sat_f':sat_f,
                'bright_f':bright_f,
                'contrast_f':contrast_f,
                'rotation_f':rotation_f,
                'zoom_f':zoom_f,
                'affine_f':affine_f,
                'translation_f':translation_f,
                'vertical_f':vertical_f}

    #need to add dictionary

    type_2 = {  'blurs':blurs,
                'noise':noise,
                'NB':NB,
                'hue':hue,
                'sat':sat,
                'bright':bright,
                'contrast':contrast,
                'rotation':rotation,
                'zoom':zoom,
                'affine':affine,
                'translation':translation,
                'vertical_flip':vertical_flip,
                'raw_images_ok':raw_images_ok}

    
    for types , val in type_1.items():
        if type(val) == float:
            pass
        else:
            raise TypeError(f'Expected type "float" for "{types}" param, got {type(val)} , check hyperparameters..')
    
    for typess , vals in type_2.items():

        if type(vals) == bool:

            pass

        else:
            raise TypeError(f'Expected type "bool" for "{typess}" param, got {type(vals)}, check hyperparameters..')
    

    try:
        #loading all the pickle files
        with open('pickle_files/coordinates_list.pickle','rb')  as ff:
            coor = pickle.load(ff)

        with open('pickle_files/images_dict.pickle','rb')  as f:
            imgs = pickle.load(f)

        with open('pickle_files/ids.pickle','rb')  as f:
            ids = pickle.load(f)

    except Exception as e:

        logger.error(f'Pickle file not found ! ,{e}')
        raise FileNotFoundError(f'Pickle file not Found!, {e}')
        

       
        
  
    start_time = time.perf_counter()
    console.print('[bold yellow]Augmentations :[/bold yellow] [b]:arrow_down:[/b]')

    

    #multiThread  #doc --> https://docs.python.org/3/library/concurrent.futures.html
    with concurrent.futures.ThreadPoolExecutor(max_workers=15) as executor: 

        if raw_images_ok:

            executor.submit(raw_image,coor,imgs,folder)
            table.add_row("Raw image", f'{str(int(len(imgs)))}/{len(imgs)}',"Default 1.0")

        if blurs:

            executor.submit(blur_im,coor,imgs,blur_f,folder)
            table.add_row("BLUR", f'{str(int(len(imgs)*blur_f))}/{len(imgs)}',str(blur_f))

        if noise:
            executor.submit(noise_im,coor,imgs,noise_f,folder)
            table.add_row("NOISE", f'{str(int(len(imgs)*noise_f))}/{len(imgs)}',str(noise_f))
          

        if NB:
           
            executor.submit(noise_blur_im,coor,imgs,NB_f,folder)
            table.add_row("NOISE AND BLUR", f'{str(int(len(imgs)*NB_f))}/{len(imgs)}',str(NB_f))

        if hue:
           
            executor.submit(hue_im,coor,imgs,hue_f,folder)
            table.add_row("HUE", f'{str(int(len(imgs)*hue_f))}/{len(imgs)}',str(hue_f))

        if sat:
      
            executor.submit(saturation_im,coor,imgs,sat_f,folder)
            table.add_row("SATURATION", f'{str(int(len(imgs)*sat_f))}/{len(imgs)}',str(sat_f))

        if bright:
          
            executor.submit(bright_dark_im,coor,imgs,bright_f,folder)
            table.add_row("Bright & dark", f'{str(int(len(imgs)*bright_f)*2)}/{len(imgs)*2}',str(bright_f)) 

        if rotation:
      
            executor.submit(rotate_90_im,coor,imgs,rotation_f,folder)
            
            executor.submit(random_rotate_im,coor,imgs,rotation_f,folder)
            table.add_row("Rotation 90deg", f'{str(int(len(imgs)*rotation_f))}/{len(imgs)}',str(rotation_f))
            table.add_row("Random rotation", f'{str(int(len(imgs)*rotation_f))}/{len(imgs)}',str(rotation_f))

        if zoom:
      
            executor.submit(zoom_im,coor,imgs,zoom_f,folder)
            table.add_row("Zoom", f'{str(int(len(imgs)*zoom_f))}/{len(imgs)}',str(zoom_f))

        if affine:
          
            executor.submit(affine_im,coor,imgs,affine_f,folder)
            table.add_row("Affine image",f'{str(int(len(imgs)*affine_f))}/{len(imgs)}',str(affine_f))
        
        if translation:
         
            executor.submit(translation_im,coor,imgs,translation_f,folder)
            table.add_row("Translation image",f'{str(int(len(imgs)*translation_f))}/{len(imgs)}',str(translation_f))
        
        if contrast:
            
            executor.submit(contrast_im,coor,imgs,contrast_f,folder)
            table.add_row("Contrast image",f'{str(int(len(imgs)*contrast_f))}/{len(imgs)}',str(contrast_f))

        if vertical_flip:
            executor.submit(vertical_im,coor,imgs,vertical_f,folder)
            table.add_row("VerticalFlip image",f'{str(int(len(imgs)*vertical_f))}/{len(imgs)}',str(vertical_f))


    console.print(f'[bold red]Time taken:[/bold red] [b]{time.perf_counter()-start_time:.2f}s[/b] :rocket:')

    with open(f'{folder}/classes.txt','a+') as classes:
        for _,value in ids.items():
            classes.write(f'{value}\n')
            

    
    
    utils_py.yaml_writer(len(ids.keys()),list(ids.values()),folder)
       


    os.remove('pickle_files/coordinates_list.pickle')
    os.remove('pickle_files/images_dict.pickle')

    console.print(f'[bold green]Images and labels[/bold green] saved in [bold blue]{folder}[/bold blue] folder.')

    #it will split the data (train , test)
    utils_py.train_test_split(folder,split=test_split)

    if table.columns:
        console.print(table)


    return 'Completed'


























# if __name__ == "__main__":

    



# # # #     aug_image = dict()
# #     saved_folder_name = 'yolo_test'
#     # raw_images = raw_image_process('yolo_denormalize.txt',folder=saved_folder_name,normalize=False)
    
#     with open('pickle_files/coordinates_list.pickle','rb')  as ff:
#             coor = pickle.load(ff)


#     with open('pickle_files/images_dict.pickle','rb')  as f:
#             imgs = pickle.load(f)

#     with open('pickle_files/ids.pickle','rb')  as f:
#             ids = pickle.load(f)

#     bb = vertical_im(coor,imgs,.8,'yolo_test')

    # cc = noise_im(raw_images,0.9,saved_folder_name)
#     dd = noise_blur_im(ra,0.9,aug_image)
#     ee = hue_im(ra,0.9,aug_image)
#     f = saturation_im(ra,0.9,aug_image)
#     g = bright_dark_im(ra,0.9,aug_image)
#     g = rotate_90_im(ra,0.9,aug_image)
    # g = zoom_im(ra,0.4,aug_image)
    # g = affine_im(ra,0.4,aug_image)
    # g = translation_im(ra,0.9,aug_image)


    # for a,b in aug_image.items():
    #         print(a)


import glob
import pybboxes as pbx
import albumentations as A
import cv2
import numpy as np
from rich.console import Console
from rich.progress import track
from rich.traceback import install

import pickle
import os
from typing import List , Tuple 

from object_detection_aug import utils_py as utils_py
from object_detection_aug import logging_util as logging_util


#logging
logger = logging_util.get_logger(os.path.basename(__file__))

#rich module
console = Console()
install()


#not in use 
def keeping_border(image:np.array,bboxes:List[List],category_ids:List,image_size:int=416) -> Tuple[np.array,List[List],List]:
    
    '''
    This function will keep border on each side of the image

    : param coordinates : coordinates is a pickle file containing coordinates 
                          of a image. coordinates contain (image_id , class_id , x,y,w,h) 
                          (load the pickle file first)
    
    : param images      : images a pickle file containing key , value pair.
                          key id of image_id (0,1,2,..) , value is cv2.imread() image(np.array)

    : param fraction    : fraction is float. by giving fraction =0.5 , it will randomly selected 50% data from your
                          dataset and apply blur to it.

    : param folder      : folder name where you want to save your augmentations.

    : return            : transfomed image , boxes coor and their ids

    '''
    transform = A.Compose(
    [A.augmentations.geometric.transforms.PadIfNeeded (p=1,min_height=image_size,min_width=image_size,border_mode=1)],
    bbox_params=A.BboxParams(format='coco', label_fields=['category_ids']))

    transformed = transform(image=image, bboxes=bboxes, category_ids=category_ids)

    return transformed['image'] , transformed['bboxes'] , transformed['category_ids']


def jpg_converter(folder:os.path) -> None:
    
    '''
    This is a important function for this code.
    if all images are not in the same format it will convert all images into jpg.

    : param folder : It will accept a folder containing annotation , images and classes.
    
    '''
  
    try:

        for images in os.listdir(folder):
            if images.endswith('.txt'):
                continue
            if images.endswith('.jpg'):
                continue
            else:
                name = images.split('.')[0]
                im = cv2.imread(f'{folder}/{images}')
                cv2.imwrite(f'{folder}/{name}.jpg',im)
                os.remove(f'{folder}/{images}')
       

    except Exception as e:

        logger.warning(f'Jpg converter folder not found or does not contain images! {e}')

        raise FileNotFoundError(f'Please provide correct folder or full path of the  folder, Provided folder: {folder}')
        
    
    
def normalize_yolo(X_tl:int,Y_tl:int,b_width:int,b_height:int,image_width:int,image_height:int) -> Tuple[float,float,float,float]:

    '''
    This function will normalize coordinates values (200,100,...,) --> (0.002,0.444,...,)

    : param image_width , image_height  : actual image height and width
    : param X_tl , Y_tl                 : top left corner
    : param b_width , b_height          : height and width of the bounding boxes

    '''

    X_tl = (X_tl+(b_width/2))/image_width
    Y_tl = (Y_tl+(b_height/2))/image_height
    b_width = b_width/image_width
    b_height = b_height/image_height

    return X_tl , Y_tl , b_width , b_height



def converter(path:os.path,resize_im:int=416,keep_aspect_ratio:bool=True,image_jpg_converter:bool=True) -> None:
    
    '''
    Converter function will convert all images into given size and save those  -> images , coordination and class_ids in pickle files.
   

    : param path              : path is your folder containing images ,annotation and classes.
    : param resize_im         : resize im is an int, it will resize images into same size
    : param keep aspect ratio : It is bool , keeping it TRUE means it will keep one side image size same. (either height and width)
    : param image_jpg_conv    : It is bool, if your images contain different extension like .PNG , .JPG , .jpeg , keeping this True means
                                it will convert all images in .jpg format. 
    
    '''
    
    remember_id = dict()
    temp_store_list = []
    temp_image = dict()
    new = []

    
    
    if type(resize_im) == int:
            pass
    else:
        raise TypeError('Please provide resize im as INT!')
    
    if type(keep_aspect_ratio) == bool:
        pass
    else:
        raise TypeError('Please provide boolean in keep asepect ratio')
    
    try:
        os.listdir(path)
        
    except:
        raise NotADirectoryError(f'{path} ...is not a directory!!')

    
    #need to add exception block
    if image_jpg_converter:
        jpg_converter(path)

    image = 0
    label = 0  
    checks = os.listdir(path)

    if checks.count('classes.txt') == 1:
        console.print('[bold green]Classes.txt found..[/bold green]')

    elif checks.count('classes.txt') > 1:
        raise Warning(f'More than "one" Classes.txt found !!')

    else:
          raise Warning(f'Classes.txt not found !!')


    for dataz in checks:
        if dataz.endswith('.jpg'):
            image += 1
        elif dataz.endswith('classes.txt'):
            continue
        elif dataz.endswith('.txt'):
            label += 1

    if (image==label) == True:

        console.print(f'[bold red]{image}[/bold red] [bold green]Images[/bold green] [b]and[/b] [bold red]{label}[/bold red] [bold green]Labels[/bold green] [b]found ![/b]')
    else:
        raise Warning(f'{image} Images and {label} Labels found!! Make sure you have same number of images and labels.')
   
    try:
        
        for paths in checks:
            if paths.endswith('classes.txt'):
        
                with open(f'{path}/{paths}')  as fff:
                    name = fff.read().splitlines()
                    console.print(f'[bold blue]Labels Name :[/bold blue] [b]{",".join(name)}[/b]')
                    for id, names in enumerate(name):
                        remember_id[id] = names
                        

    except Exception as e:
        
        logger.error('Classes.txt not found!')
        raise FileNotFoundError(f'Please provide full folder path,{e}')
    
    for i , data in enumerate(track(glob.glob(f'{path}/*txt'),description='Creating pickle file')):

        

        try:
       
            image = data.split('.')
            
        
            if keep_aspect_ratio:

                try:
                    if image[0].endswith('classes'):
                        continue

                    read_image = utils_py.image_resize(f'{image[0]}.jpg',image_size=resize_im)

                    temp_image[i] = read_image

                    

                    if read_image is None:
                        continue
        
                    h,w,c = read_image.shape

                    with open(f'{data}','r')  as ff:

                        split= ff.read().splitlines()

                        len_split = len(split)

                        for z , s in enumerate(range(len(split))):
                        
                            res = split[s]
                            
                            id = res.split()[0]# --> class id

                            coor = tuple(map(float,res.split()[1:])) # --> coordinates

                            
                            x_tl , y_tl , width , height = pbx.convert_bbox(coor, from_type="yolo", to_type="coco",image_size=(w,h)) 
                            
                            temp_store_list.insert(0,(i,id,x_tl,y_tl,width,height)) #need to write doc

                        
                        new.append(temp_store_list[0:len_split]) ###

                except Exception as e:
                    logger.warning(f'Exact data is not present in the folder, {e}')
                    raise Warning(f'data is missing in {path} ')


           
                  
                
        except Exception as e:
            logger.warning('Conveter can not convert all data.')
            raise FileNotFoundError(f'Please provide full path of {path}')

    if not os.path.exists('pickle_files'):
        os.makedirs('pickle_files')
        
    with open('pickle_files/ids.pickle','wb') as pick:
        pickle.dump(remember_id,pick) 

    with open('pickle_files/images_dict.pickle','wb') as p:
        pickle.dump(temp_image,p,protocol=pickle.HIGHEST_PROTOCOL) 
    
    with open('pickle_files/coordinates_list.pickle','wb') as pickss:
        pickle.dump(new,pickss)
    
    del remember_id 
    del temp_store_list
    del temp_image
    del new 

    return 'Done'

                
            


     
        
        



# if __name__ == '__main__':
#     output = converter('/home/souviks/Documents/codelogicx/l-pesa/flask/pass_mrz/augmentations_for_yolo_1/card_gens',keep_aspect_ratio=True,image_jpg_converter=True,resize_im=640)
#     print(output)














from asyncio.log import logger
import random
import os
from typing import List


def read_txt(filename:os.path) -> os.path:
    '''
    This function will open txt file and shuffle the data

    '''
    lines = open(filename).readlines()
    random.shuffle(lines)
    return lines


def fraction_data(file:os.path,fraction:float=.9) -> List:

    '''
    This function will create a fraction of data,suppose you have 100 images and you want to add blur only
    on randomly selected 20 images, by passing fraction param = 0.2, it will randomly select 20 images from 
    dataset and apply blur to it, for selecting all images you have to pass fraction = 1.0 

    : param file             : it is a list
    : param fraction         : fraction float value
    : return random_sampling : It will return a list of cv2.imread() images(array)
    
    '''
    try:
        percentage = int(len(file)*fraction)
        random_sampling =random.sample(file,percentage)
        return random_sampling

    except Exception as e:
        logger.error(f'Fraction data creating issue, {e}')


from tornado import concurrent
import cv2
import uuid
from imgaug.augmentables.polys import Polygon, PolygonsOnImage
from rich.console import Console
from rich.traceback import install
from rich.progress import track

import json
import os
import glob
import shutil
from typing import Dict , Tuple

from instance_seg.augment_poly import *
from instance_seg import utils_poly
from instance_seg import yml_writer_poly
import instance_seg.logging_util as logging_util


install()
console = Console()
logger = logging_util.get_logger(os.path.basename(__file__).split('.')[0])


'''
Code logic starts from here.

'''

class PolygonAugmentation():
    def __init__(self,aug_save_folder_name:os.path='polygon_augmentation',image_resize:int=640) -> None:
        self.aug_save_folder_name = aug_save_folder_name
        self.augment = ImageAugmentation(image_resize)
        self.store_dict:dict = dict()
        self.counter:int = 0
        
        
        if os.path.exists(f'{self.aug_save_folder_name}'):
            raise NotImplementedError(f'"{self.aug_save_folder_name}" folder already exist, please change your augmentation saved folder name..')

        if not os.path.exists(f'{self.aug_save_folder_name}/train/images') or not os.path.exists(f'{self.aug_save_folder_name}/train/labels'):
            os.makedirs(f'{self.aug_save_folder_name}/train/images')
            os.makedirs(f'{self.aug_save_folder_name}/train/labels')
            console.print(f'[bold blue] [+] {self.aug_save_folder_name}/[bold blue] folder - created..')
        
        if not os.path.exists(f'{self.aug_save_folder_name}/test/images') or not os.path.exists(f'{self.aug_save_folder_name}/test/labels'):
            os.makedirs(f'{self.aug_save_folder_name}/test/images')
            os.makedirs(f'{self.aug_save_folder_name}/test/labels')
            
            
        self.train_images_path = f'{self.aug_save_folder_name}/train/images'
        self.train_labels_path = f'{self.aug_save_folder_name}/train/labels'
        self.test_images_path = f'{self.aug_save_folder_name}/test/images'
        self.test_labels_path = f'{self.aug_save_folder_name}/test/labels'
    
        logger.info('ImageAugments module loaded...')

    def json_converter(self,image:os.path):
        '''
        Json converter will convert json to polygon acceptable format.
        It will return image and converted json

        : param image : require full path of a image 
        
        : return :
        : im_read       : it will return cv2.imread() image.
        : poly on image : it will return json converted polygon points 
        
        
        '''
        try:
            # temporary to store the result
            datas = []
            # first we are checking the extension endswith .json or not 
            # then we read the image and json file
            if not image.endswith('.json'):
                    im_read = cv2.imread(image)
        
                    JSON = image.split('.')[0] + '.json'
                    with open(JSON) as f:
                        data = json.load(f)
                        annotations = data['shapes']
                        for i ,annotation in enumerate(annotations):
                            ids = uuid.uuid4()
                            self.labels = annotation['label']
                            
                            if self.labels in self.store_dict:
                                pass
                            
                            else:
                                self.store_dict[self.labels] = self.counter
                                self.counter += 1
                                
                            poly_points = annotation['points']
                            poly_points = [tuple(lst) for lst in poly_points]
                            ids = Polygon(poly_points,self.labels)
                            datas.append(ids)
                            
                        poly_on_image = PolygonsOnImage(datas, shape=im_read.shape)
                        
                        del datas
                    
                        yield im_read , poly_on_image

        except Exception as e:
            logger.error(f'problem : Json converter  desc : {e}')       
                   
                  
    def Combined_augmentation(self,im_read , poly_on_image, no_track, blur=True , blur_f = 0.5 , rotate=True , rotate_f = 0.5 , noise=True,noise_f=0.5,perspective=True,perspective_f = 0.5,affine=True,affine_f=0.5,
                              brightness=True,brightness_f=0.5,hue=True,hue_f=0.5,removesaturation=True,removesaturation_f=0.5,contrast=True,contrast_f=0.5,upflip=True,upflip_f=0.5,
                              shear=True ,shear_f=0.5, rotate90=True,rotate90_f = 0.5,blur_and_noise=True,blur_and_noise_f=0.5,image_cutout = True,image_cutout_f=0.5,
                              mix_aug=True,mix_aug_f=0.5,temperature_change=True,temperature_change_f=0.5):
        
        
        
        with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:
            
            if blur:
           
                frac_data  = int(self.split * blur_f)
                if no_track <= frac_data:
                    points =  executor.submit(self.augment.image_blur,im_read,poly_on_image)
                    p , im = next(points.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
            
            if rotate:
                frac_data  = int(self.split * rotate_f)
                if no_track <= frac_data:
                    points2  = executor.submit(self.augment.image_rotate,im_read,poly_on_image) 
                    p , im = next(points2.result())
                    utils_poly.create_new_txt(im,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if noise:
                frac_data  = int(self.split * noise_f)
                if no_track <= frac_data:
                    points3  = executor.submit(self.augment.image_noise,im_read,poly_on_image) 
                    p , im = next(points3.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if perspective:
                frac_data  = int(self.split * perspective_f)
                if no_track <= frac_data:
                    points4  = executor.submit(self.augment.image_perspective_transform,im_read,poly_on_image) 
                    p , im = next(points4.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if affine:
                frac_data  = int(self.split * affine_f)
                if no_track <= frac_data:
                    points5  = executor.submit(self.augment.image_affine,im_read,poly_on_image) 
                    p , im = next(points5.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if brightness:
                frac_data  = int(self.split * brightness_f)
                if no_track <= frac_data:
                    points6  = executor.submit(self.augment.image_brightness,im_read,poly_on_image) 
                    p , im = next(points6.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
            
            if hue:
                frac_data  = int(self.split * hue_f)
                if no_track <= frac_data:
                    points6  = executor.submit(self.augment.image_hue,im_read,poly_on_image) 
                    p , im = next(points6.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if removesaturation:
                frac_data  = int(self.split * removesaturation_f)
                if no_track <= frac_data:
                    points7  = executor.submit(self.augment.image_removeSaturation,im_read,poly_on_image) 
                    p , im = next(points7.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if contrast:
                frac_data  = int(self.split * contrast_f)
                if no_track <= frac_data:
                    points8 = executor.submit(self.augment.image_contrast,im_read,poly_on_image) 
                    p , im = next(points8.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if upflip:
                frac_data  = int(self.split * upflip_f)
                if no_track <= frac_data:
                    points9  = executor.submit(self.augment.image_upFlip,im_read,poly_on_image) 
                    p , im = next(points9.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                    
            if shear:
                frac_data  = int(self.split * shear_f)
                if no_track <= frac_data:
                    points10  = executor.submit(self.augment.image_shear,im_read,poly_on_image) 
                    p , im = next(points10.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if rotate90:
                frac_data  = int(self.split * rotate90_f)
                if no_track <= frac_data:
                    points11  = executor.submit(self.augment.image_rotate90,im_read,poly_on_image) 
                    p , im = next(points11.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if blur_and_noise:
                frac_data  = int(self.split * blur_and_noise_f)
                if no_track <= frac_data:
                    points12  = executor.submit(self.augment.blur_and_noise,im_read,poly_on_image) 
                    p , im = next(points12.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                
            if image_cutout:
                frac_data  = int(self.split * image_cutout_f)
                if no_track <= frac_data:
                    points13  = executor.submit(self.augment.image_cutOut,im_read,poly_on_image) 
                    p , im = next(points13.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
            
            if mix_aug:
                frac_data  = int(self.split * mix_aug_f)
                if no_track <= frac_data:
                    points14  = executor.submit(self.augment.mixed_aug_1,im_read,poly_on_image) 
                    p , im = next(points14.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                    
                    points15  = executor.submit(self.augment.mixed_aug_2,im_read,poly_on_image) 
                    p , im = next(points15.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                    
                    points16  = executor.submit(self.augment.mixed_aug_3,im_read,poly_on_image) 
                    p , im = next(points16.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
                    
                    points17  = executor.submit(self.augment.mixed_aug_4,im_read,poly_on_image) 
                    p , im = next(points17.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
            
            if temperature_change:
                frac_data  = int(self.split * temperature_change_f)
                if no_track <= frac_data:
                    points18  = executor.submit(self.augment.image_change_colorTemperature,im_read,poly_on_image) 
                    p , im = next(points18.result())
                    utils_poly.create_new_txt(im ,self.labels,p,self.train_images_path,self.train_labels_path,self.store_dict)
             
  
               
    
    def Image_augmentation(self,folder,train_split=1.0,blur=True , blur_f = 0.5 , rotate=True , rotate_f = 0.5 , noise=True,noise_f=0.5,perspective=True,perspective_f = 0.5,affine=True,affine_f=0.5,
                            brightness=True,brightness_f=0.5,hue=True,hue_f=0.5,removesaturation=True,removesaturation_f=0.5,contrast=True,contrast_f=0.5,upflip=True,upflip_f=0.5,
                            shear=True ,shear_f=0.5, rotate90=True,rotate90_f = 0.5,blur_and_noise=True,blur_and_noise_f=0.5,image_cutout = True,image_cutout_f=0.5,
                            mix_aug=True,mix_aug_f=0.5,temperature_change=True,temperature_change_f=0.5):
        
        dir_checker = os.path.isdir(folder)
        
        if dir_checker != True:
            shutil.rmtree(self.aug_save_folder_name)
            raise NotADirectoryError(f'Provided Path - {folder} is not a directory...')
       
        # changing other images format to same format
        self.image_format_change(folder=folder)
        
        all_images =  glob.glob(f'{folder}/*jpg')
        all_json =  glob.glob(f'{folder}/*json')
        
        if len(all_images) == len(all_json) and len(all_images) >= 1 and len(all_json) >=1:
            del all_json
            
        else:
            
            raise NotImplementedError(f'Images and Jsons are not equal , recheck your annotation folder! \
                                       Total Images : {len(all_images)}  |  Total Json : {len(all_json)}')
        
        if type(train_split) != float:
            raise TypeError(f'please provide "train split" as "float" , You provided train split as {type(train_split)}')
            
            
        if train_split > 1.0:
            raise ValueError(f'[-] please provide "train split" between "0.5 to 1.0", Your provided train split value is : {train_split}')

            
        type_1 = { 
                'blur_f':blur_f,
                'noise_f':noise_f,
                'Noise_and_blur_f':blur_and_noise_f,
                'hue_f':hue_f,
                'removeSaturation_f':removesaturation_f,
                'bright_f':brightness_f,
                'contrast_f':contrast_f,
                'rotation_f':rotate_f,
                'rotation90_f':rotate90_f,
                'affine_f':affine_f,
                'perspective_f':perspective_f,
                'upflip_f' : upflip_f,
                'shear_f' : shear_f,
                'image_cut_f':image_cutout_f,
                'mix_aug_f' : mix_aug_f,
                'temperaure_change_f' : temperature_change_f
                
              }
        
        type_2 = { 
                'blur':blur,
                'noise':noise,
                'NB':blur_and_noise,
                'hue':hue,
                'removeSat':removesaturation,
                'bright':brightness,
                'contrast':contrast,
                'rotation':rotate,
                'rotation90':rotate90,
                'affine':affine,
                'perspective':perspective,
                'upflip' : upflip,
                'shear' : shear,
                'image_cut':image_cutout,
                'mix_aug' : mix_aug,
                'temp_change' : temperature_change
                
              }
            
        for types , val in type_1.items():
            if type(val) != float:
                raise TypeError(f' [-] please provide "{types}" as  "float" , You provided "{types}" as {type(val)}')
             
                
            if val > 1.0:
                raise ValueError(f'[-] please provide "{types}" value  between "0.1 to 1.0" , Your provided "{types}" value is : "{val}"')
        
        for typess , vals in type_2.items():

            if type(vals) != bool:
                raise TypeError(f'Please provide "{typess}" as bool , you provided "{typess}" as "{vals}"')
        
        
        
        
        
        random.shuffle(all_images)
        
        self.len_total_images = len(all_images)
        
        self.split = int(self.len_total_images * train_split)
        
        console.print(f'[bold cyan] [+] Total images : {self.len_total_images}   |   Train Split : {self.split} images   |    Test split : {self.len_total_images-self.split} images [bold cyan]')
        
        if train_split == 1.0:
                    shutil.rmtree(f'{self.aug_save_folder_name}/test')
        
                    
        for c ,images in enumerate(track(all_images,description='Image Augmentation..',)):

            if c+1 <= self.split:
                
                poly = list(self.json_converter(images))
                
                self.Combined_augmentation(poly[0][0],poly[0][1],c+1,blur=blur , blur_f = blur_f , rotate=rotate , rotate_f = rotate_f , noise=noise,noise_f=noise_f,perspective=perspective,perspective_f = perspective_f,affine=affine,affine_f=affine_f,
                              brightness=brightness,brightness_f=brightness_f,hue=hue,hue_f=hue_f,removesaturation=removesaturation,removesaturation_f=removesaturation_f,contrast=contrast,contrast_f=contrast_f,upflip=upflip,upflip_f=upflip_f,
                              shear= shear ,shear_f=shear_f, rotate90=rotate90,rotate90_f = rotate90_f,blur_and_noise=blur_and_noise,blur_and_noise_f=blur_and_noise_f,image_cutout = image_cutout,image_cutout_f=image_cutout_f,
                              mix_aug=mix_aug,mix_aug_f=mix_aug_f,temperature_change=temperature_change,temperature_change_f=temperature_change_f)
                
                
                
            elif c+1 > self.split:
                # print(images)
                
                poly = list(self.json_converter(f'{images}'))
                utils_poly.create_new_txt(poly[0][0] ,self.labels,poly[0][1],self.test_images_path,self.test_labels_path,self.store_dict)
                   
        console.print(f'[bold dim cyan] Labels name : [/bold dim cyan] [bold magenta] {list(self.store_dict.keys())} [bold magenta]')
        yml_writer_poly.yaml_writer(len(self.store_dict.keys()),list(self.store_dict.keys()),self.aug_save_folder_name)      
      
      
      
      
    @staticmethod
    def image_format_change(folder):
        for im in os.listdir(folder):
            if im.endswith('.json'):
                continue
            elif im.endswith('.jpg'):
                continue
            else:
                
                im_name = im.split('.')[0]
                ims = cv2.imread(f'{folder}/{im}')
                cv2.imwrite(f'{folder}/{im_name}.jpg',ims)
                os.remove(f'{folder}/{im}')
                      

         


    










  
        



import logging
import os
from time import strftime
from typing import List, Union , TypeVar


time_now = strftime("%d-%m-%Y")

WARNING = TypeVar('WARNING')
ERROR = TypeVar('ERROR')
INFO = TypeVar('INFO')
DEBUG = TypeVar('DEBUG')


def get_logger(base_name:os.path.basename(__file__)) -> logging.Logger:

    '''
    Get_logger will take python script name (base_name) Ex : os.path.basename(__file__) , and mark it as base_name for formatter.
    It will be helpful for Debugging.
    Get logger will create a log folder & save the logs_(%d%M%Y) there.

    : param base_name : base_name takes python script name , Ex : os.path.basename(__file__).split('.')[0] --> logging.py --> logging 
    : return          : logger

    '''

    if not os.path.exists('logs'):
        os.makedirs('logs')
        
    
    logger = logging.getLogger(base_name)
    logger.setLevel(logging.INFO)
    format = logging.Formatter(f'%(asctime)s - %(levelname)s - {base_name} - line - %(lineno)d ==> %(message)s')
    file_name = logging.FileHandler(f'logs/log_{time_now}.log',mode='a',delay=False)
    file_name.setFormatter(format)
    logger.addHandler(file_name)

    return logger
    


def log_file_reader(file_name:os.PathLike,level_name:Union[WARNING,ERROR,INFO,DEBUG]='WARNING') -> List:

    '''
    Log file reader will reads log files according to log_level_name , default set to 'WARNING'.
    
    : param log_file_name : Expects .log format file path.
    : param level name    : Log message show according to its level name , default 'WARNING'.
    : return              : It will return list of specific log messages.


    '''

    assert file_name.split('.')[1] == 'log' ,'please provide correct format... (.log) file'

    with open(file_name,'r') as file_reader:
        reads = file_reader.readlines()
        filter_log = [line for line in reads if level_name in line]   
        return filter_log
   
    

  




# if __name__ == '__main__':
    # reads = log_file_reader('logs/log_25-07-2022.log',)
#     for t in reads:
#         print(t)

